//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// An object holding all of the information on the data
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
class DataObject{

    Point     _geometry[];     // The geometry
    Triangle  _connect[];      // The connectivity of the geometry  
    double    _histogram[][];  // The histogram
    double    _minDatas[];     // The minimum data values for each point
    double    _maxDatas[];     // The maximum data values for each point
    double    _minDensities[]; // The minimum density values for each pt
    double    _maxDensities[]; // The maximum density values for each pt
    double    _means[];        // The mean data value for each point
    double    _stdDevs[];       // The std deviation for each point

    // * * * Constructor * * * //
    DataObject(){}

    // * * * Return the number of bins * * * //
    int getNumBins() { 
	if(_histogram == null)    return 256; 
	else if(_histogram[0] == null)  return 256; 
	else return _histogram[0].length; 
    }

    // * * * Return the number of points * * * //
    int getNumPts() { if(_geometry == null) return -1; else return _geometry.length; }

    // * * * Return the number of triangles * * * //
    int getNumTris() { return _connect.length; }

    // * * * Return the point * * * //
    Point getGeometry(int i) {
	if(i >= 0 && i < getNumPts()) return _geometry[i]; 
	else{ System.out.println("ERROR dataObject::getGeometry INDEX OUT OF BOUNDS!"); return new Point(); }
    }

    // * * * Return the triangle * * * //
    Triangle getConnect(int i) {	
	if(i >= 0 && i < getNumTris()) return _connect[i]; 
	else{ System.out.println("ERROR dataObject::getConnect INDEX OUT OF BOUNDS!"); return new Triangle(); }
    }

    // * * * Return the histogram * * * //
    double [][] getHistogram() { return _histogram; }

    // * * * Get a pt from the histogram * * * //
    double [] getHistogram(int i) {	
	if(i >= 0 && i < getNumPts()) return _histogram[i]; 
	else{ System.out.println("ERROR dataObject::getHistogram INDEX OUT OF BOUNDS!"); return new double[0]; }
    }

    // * * * Return the max density * * * //
    double getMaxDensity(int i) { 
	if(i >= 0 && i < getNumPts()) return _maxDensities[i];
	else{ System.out.println("ERROR dataObject::getMaxDensity INDEX OUT OF BOUNDS!"); return 0.0; }
    }

    // * * * Return the min density * * * //
    double getMinDensity(int i) { 
	if(i >= 0 && i < getNumPts()) return _minDensities[i];
	else{ System.out.println("ERROR dataObject::getMinDensity INDEX OUT OF BOUNDS!"); return 0.0; }
    }

    // * * * Get the max data * * * //
    double getMaxData(int i) { 
	if(i >= 0 && i < getNumPts()) return _maxDatas[i];
	else { System.out.println("ERROR dataObject::getMaxData INDEX OUT OF BOUNDS!"); return 0.0; }
    }

    // * * * Get the min data * * * //
    double getMinData(int i) { 
	if(i >= 0 && i < getNumPts()) return _minDatas[i];
	else { System.out.println("ERROR dataObject::getMinData INDEX OUT OF BOUNDS!"); return 0.0; }
    }

    // * * * Get all of the mins *  * * //
    double [] getMinDatas() { return _minDatas; }

    // * * * Get all of the maxs * * * //
    double [] getMaxDatas() { return _maxDatas; }

    // * * * Get the mean at pt i * * * // 
    double getMean(int i) {
	if(i >= 0 && i < getNumPts()) return _means[i];
	else { System.out.println("ERROR dataObject::getMean INDEX OUT OF BOUNDS!"); return 0.0; }
    }

    // * * * Get all of the means * * * //
    double [] getMeans() { return _means; }

    // * * * Get the standard deviation at i * * * //
    double getStdDev(int i) {
	if(i >= 0 && i < getNumPts()) return _stdDevs[i];
	else { System.out.println("ERROR dataObject::getStdDev INDEX OUT OF BOUNDS!"); return 0.0; }
    }

    // * * * Get all of the std devs * * * //
    double [] getStdDevs() { return _stdDevs; }
    
    // * * * Set the geometry * * * //
    void setGeometry(Point [] geometry) { 
	_geometry = geometry;
	_histogram = new double[getNumPts()][];
	_minDatas = new double[getNumPts()];     _maxDatas = new double[getNumPts()]; 
	_minDensities = new double[getNumPts()]; _maxDensities = new double[getNumPts()];
	_means = new double[getNumPts()];
	_stdDevs = new double[getNumPts()];
    }

    // * * * Set the connectivity * * * //
    void setConnectivity(Triangle [] connect) { _connect = connect; }

    // * * * Set the histogram * * * //
    void setHistogram(double [][] histogram) { _histogram = histogram; }   

    // * * * Set the histogram * * * //
    void setHistogram(int i, double [] histo) {
	if(i >= 0 && i < getNumPts()) _histogram[i] = histo;
	else  System.out.println("ERROR dataObject::setHistogram INDEX OUT OF BOUNDS!");
    }

    // * * * Set the max data * * * //
    void setMaxData(int i, double data) { 
	if(i >= 0 && i < getNumPts()) _maxDatas[i] = data;
	else System.out.println("ERROR dataObject::setMaxData INDEX OUT OF BOUNDS!"); 
    }

    // * * * Set the min data * * * //
    void setMinData(int i, double data) { 
	if(i >= 0 && i < getNumPts()) _minDatas[i] = data;
	else  System.out.println("ERROR dataObject::setMinData INDEX OUT OF BOUNDS!");
    }

    // * * * Set the max density * * * //
    void setMaxDensity(int i, double data) { 
	if(i >= 0 && i < getNumPts()) _maxDensities[i] = data;
	else System.out.println("ERROR dataObject::setMaxDensity INDEX OUT OF BOUNDS!"); 
    }

    // * * * Set the min density * * * //
    void setMinDensity(int i, double data) { 
	if(i >= 0 && i < getNumPts()) _minDensities[i] = data;
	else  System.out.println("ERROR dataObject::setMinDensity INDEX OUT OF BOUNDS!");
    }
    
    // * * * Set the mean * * * //
    void setMean(int i, double mean) {
	if(i >= 0 && i < getNumPts()) _means[i] = mean;
	else  System.out.println("ERROR dataObject::setMean INDEX OUT OF BOUNDS!");
    }

    // * * * Set the standard deviation * * * //
    void setStdDev(int i, double stdDevs){
	if(i >= 0 && i < getNumPts()) _stdDevs[i] = stdDevs;
	else  System.out.println("ERROR dataObject::setStdDev INDEX OUT OF BOUNDS!");
    }

    // * * * Get the closest point * * * //
    Point getClosestPt(Point pt) {

	if(getNumPts() > 0) {
	    float minDist = pt.distance(_geometry[0]);
	    Point retPt = _geometry[0];
	    for(int i = 1; i < _geometry.length; i++) {
		float tmpDist = pt.distance(_geometry[i]);
		if(tmpDist < minDist) { minDist = tmpDist; retPt = _geometry[i]; retPt.idx(i); }
	    }
	    return retPt;
	}
	else return new Point(0,0,0);
    }
}
