//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Class to read in a folder with the data
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
class DataReader{
    String _folderPath;
    
    // * * * Constructor sets the folde path for the data * * * //
    DataReader(String folderPath){ _folderPath = folderPath; }
    
    // * * * Read in the geometry, get the number of points * * * //
    Point [] readGeometry(){
	String filename = _folderPath+"/geometry.txt";
	String lines[] = loadStrings(filename);
	int numPts = lines.length;
	Point [] geometry = new Point[numPts];
	for(int i = 0; i < numPts; i++) {
	    String[] pieces = splitTokens(lines[i]);
	    geometry[i] = new Point(float(pieces[0]), float(pieces[1]), float(pieces[2]));
	}
	return geometry;
    }

    // * * * Read in the connectivity * * * //
    Triangle [] readConnectivity(){
	String filename = _folderPath+"/connectivity.txt";
	String lines[] = loadStrings(filename);
	int numTris = lines.length;
	Triangle [] connect = new Triangle[numTris];
	for(int i = 0; i < numTris; i++) {
	    String[] pieces = splitTokens(lines[i]);
	    connect[i] = new Triangle(int(float(pieces[0])), int(float(pieces[1])), int(float(pieces[2])));
	}
	return connect;
    }

    // * * * Process the data into a histogram* * * //
    void generateHistogram(int numBins, DataObject object){

	int numPts = object.getNumPts();
	print("Generating histogram");
	// Read in the solution line by line
	BufferedReader reader;
	reader = createReader(_folderPath+"/data.txt");
	String line = "";
	int pointCount = 0;

	while (line != null) {
	    try { line = reader.readLine(); } 
	    catch (IOException e) { e.printStackTrace(); line = null; }
	    if (line != null) {
		print(".");
		String slist[] = split(trim(line), ' ');
	    double list[] = new double[slist.length];
	    int i=0;
            for(String s:slist){
		list[i] = Double.parseDouble(s);
                i++;
	    }
	    calculateHistogram(numBins, list, object, pointCount);
	    pointCount ++;
	    }
	}
	println("done.");
    }

    // * * * Calcuate the histogram, record the min/maxs
    void calculateHistogram(int numBins, double [] data, DataObject object, int pt){
	double [] histogram = new double[numBins];
	double mean = 0.0; double stdDev = 0.0;

	// Find the min/max of the data
	double min = data[0]; double max = data[0];
	for(int i = 0; i < data.length; i++) {
	    min = min < data[i] ? min : data[i];
	    max = max > data[i] ? max : data[i];
	    mean += data[i];			  
	}
	mean /= data.length;
	if(min == max) mean = min;
	object.setMean(pt, mean);
	object.setMinData(pt, min);
	object.setMaxData(pt, max);

	// Count the number of data vals that fall in each bin
	for(int i = 0; i < data.length; i++) {	
            
	    double findex = Etc.affine(min, (double)data[i], max, 0.0d, (double)(numBins-1));
	    int index = (int)Math.floor(findex);
	    histogram[index] ++;
	    stdDev += (data[i]-mean)*(data[i]-mean);
	}
	stdDev = Math.sqrt( stdDev/(data.length-1) );
	object.setStdDev(pt, stdDev);

	// Normalize the count
	for(int i = 0; i < numBins; i++) {          
	    if(min == max) histogram[i] = 1.0/(numBins);
            else           histogram[i] = histogram[i]/(data.length);
	}
	histogram[numBins-1] = histogram[numBins-2];
	object.setHistogram(pt, histogram);
	
	// Get the min/max densities
	double minDense = histogram[0];
	double maxDense = histogram[0];
	for(int i = 0; i < numBins; i++) {
	    minDense = minDense < histogram[i] ? minDense : histogram[i];
	    maxDense = maxDense > histogram[i] ? maxDense : histogram[i];
	}
	object.setMaxDensity(pt, maxDense);
	object.setMinDensity(pt, minDense);
    }
}

